package NGCP::Panel::Controller::API::VoicemailRecordingsItem;
use NGCP::Panel::Utils::Generic qw(:all);

use Sipwise::Base;

use HTTP::Headers qw();
use HTTP::Status qw(:constants);

use NGCP::Panel::Utils::ValidateJSON qw();
use NGCP::Panel::Utils::Subscriber;
use NGCP::Panel::Utils::Sounds;
require Catalyst::ActionRole::ACL;
require NGCP::Panel::Role::HTTPMethods;
require Catalyst::ActionRole::RequireSSL;

sub allowed_methods{
    return [qw/GET OPTIONS HEAD/];
}

use parent qw/Catalyst::Controller NGCP::Panel::Role::API::VoicemailRecordings/;

sub resource_name{
    return 'voicemailrecordings';
}
sub dispatch_path{
    return '/api/voicemailrecordings/';
}
sub relation{
    return 'http://purl.org/sipwise/ngcp-api/#rel-voicemailrecordings';
}

__PACKAGE__->config(
    action => {
        map { $_ => {
            ACLDetachTo => '/api/root/invalid_user',
            AllowedRole => [qw/admin reseller subscriberadmin subscriber/],
            Args => 1,
            Does => [qw(ACL RequireSSL)],
            Method => $_,
            Path => __PACKAGE__->dispatch_path,
        } } @{ __PACKAGE__->allowed_methods }
    },
);

sub gather_default_action_roles {
    my ($self, %args) = @_; my @roles = ();
    push @roles, 'NGCP::Panel::Role::HTTPMethods' if $args{attributes}->{Method};
    return @roles;
}

sub auto :Private {
    my ($self, $c) = @_;

    $self->set_body($c);
    $self->log_request($c);
    return 1;
}

sub GET :Allow {
    my ($self, $c, $id) = @_;
    {
        last unless $self->valid_id($c, $id);
        my $item = $self->item_by_id($c, $id);
        last unless $self->resource_exists($c, voicemailrecording => $item);
        my $format = $c->request->params->{format} // 'wav';
        unless ($format && $format =~ /^(wav|mp3|ogg)$/) {
            $self->error($c, HTTP_INTERNAL_SERVER_ERROR, "Unknown format '$format', supported: wav,mp3,ogg.");
            last;
        }
        my $sr = \%NGCP::Panel::Utils::Subscriber::;
        my $ss = \%NGCP::Panel::Utils::Sounds::;
        my $filename = $sr->{get_voicemail_filename}($c,$item,$format);
        $c->response->header ('Content-Disposition' => 'attachment; filename="'.$filename.'"');
        $c->response->content_type($sr->{get_voicemail_content_type}($c,$format));
        $c->response->body($ss->{transcode_data}($item->recording,'WAV',uc($format)));
        return;
    }
    return;
}

sub HEAD :Allow {
    my ($self, $c, $id) = @_;
    $c->forward(qw(GET));
    $c->response->body(q());
    return;
}

sub OPTIONS :Allow {
    my ($self, $c, $id) = @_;
    my $allowed_methods = $self->allowed_methods_filtered($c);
    $c->response->headers(HTTP::Headers->new(
        Allow => join(', ', @{ $allowed_methods }),
        Accept_Patch => 'application/json-patch+json',
    ));
    $c->response->content_type('application/json');
    $c->response->body(JSON::to_json({ methods => $allowed_methods })."\n");
    return;
}

sub end : Private {
    my ($self, $c) = @_;

    #$self->log_response($c);
    return;
}


# vim: set tabstop=4 expandtab:
